:
# NAME:
#	dns_convert.sh - convert DNS setup 
#
# SYNOPSIS:
#	dns_convert.sh [--scm "SCM"] [secondary]
#
# DESCRIPTION:
#	This script does everything.  It reads /etc/named.conf and
#	creates a new named.conf in the current directory which
#	is assumed to be a temporary directory prior to importing the
#	result into an SCM (git, hg, cvs).
#
#	Without the 'secondary' argument, it reads /etc/named.conf and 
#	for each primary zone, it creates the datafile that the .soa
#	file will expect to $INCLUDE (see
#	http://www.crufty.net/help/dns/DNSMagic.htm)
#	and copy the data from the old zone file into it while removing the
#	SOA record and adding an RCSid.
#
#	It then runs dnsdeps -N named.conf to see what files still
#	need to be copied from the old DNS tree to the new, and
#	fetches them.  This step is repeated until no files are
#	missing or it becomes obvious something is wrong.
#	
#	At the end of the exercise, you should have a new named.conf
#	plus populated hosts and rev directories ready for import into
#	your SCM.
#
#	You should edit .updsoa.cf or .dns.cf so that your .soa 
#	files will have the correct nameserver and mail address
#	in them.
#	For example I have the following in .dns.cf:
#.nf
#		NS_LIST = ns.list
#		ns_list [=] ns.crufty.net, ns2.crufty.net
#		hostmaster = hostmaster.crufty.net
#		# for secondary-zones
#		primaries [=] 192.168.66.6
#
#.fi
#
#	'ns_list' is a list of name servers, the first element will go
#	in SOA records, and 'hostmaster' is in the odd format DNS
#	wants.
#	The 'primaries' list is for secondary zones.
#	
#	The 'secondary' invocation is intended for running after the
#	new DNS data has been imported into the SCM and checked out under
#	/var/named and will re-read /etc/named.conf, and copy each
#	secondary zone file from its old location to the new (renaming
#	it as necessary). 
#
#	To cut over to the new setup, just make /etc/named.conf
#	a symlink to /var/named/named.conf and restart named.
#
# BUGS:
#	Converting a BIND-9 named.conf is handled by first
#	generating an equivalent (more or less) named.boot file.
#
#	BIND-9 needs a controls section for rndc(8), we assume
#	rndc-confgen(8) is used to generate /etc/rndc.key
#
# SEE ALSO:
#	cvs(1), dns_convzone.sh(8), dnsdeps.pl(8).
#		
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: dns_convert.sh,v 1.25 2023/06/12 18:41:58 sjg Exp $
#
#	@(#) Copyright (c) 1997-2022 Simon J. Gerraty.
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

NDCONF=${NDCONF:-$DESTDIR/etc/named.conf}

test -s $NDCONF || exit 0

Mydir=`dirname $0`

while :
do
	case "$1" in
	--scm) SCM=$2; shift 2;;
	*=*) eval "$1"; shift;;
	*) break;;
	esac
done

SCM=${SCM:-git}
BINDVERS=${BINDVERS:-9}
case "$NDCONF" in
*.conf)
	# this just simplifies the parsing below
	sh $Mydir/ndconv.sh $NDCONF > ndb.$$
	NDBOOT=ndb.$$
	OUTFILE=${OUTFILE:-named.conf}
	;;
esac

case "$1" in
sec*)	OUTFILE=/dev/null
	SECONDARY=
	PRIMARY=:
	;;
*)
	OUTFILE=${OUTFILE:-`basename $NDBOOT`}
	SECONDARY=:
	PRIMARY=
	;;
esac

PZONES=primary.zones
BZONES=backup.zones
OPTS=opts.$$

# pick up the routines we need.
. $Mydir/dns_convzone.sh

# cleanup
trap 'rm -f .env .missing *.$$' 0

case "$1" in
sec*)
	conv_zones < $NDBOOT > /dev/null
	exit 0
	;;
esac

for f in Makefile GNUmakefile dnsmagic.mk
do
	test -s $Mydir/../$f && cp $Mydir/../$f .
done
ignore=.${SCM}ignore
if test ! -s $ignore; then
	cp $Mydir/../scm.ignore $ignore

	case "$SCM" in
	cvs)
		cp $ignore hosts
		cp $ignore rev
		;;
	esac
fi

> .env
> $OUTFILE
	
for f in $PZONES $BZONES
do
	mkhead > $f
done
cat <<!EOH! > $OPTS
	directory "${DNSDIR:-/var/named}";
	// we keep the pid file local to ensure running
	// multiple nameservers does not cause problems.
	pid-file "named.pid";
	
	// uncomment query-source if you need to traverse a firewall...
	// query-source address * port 53;

!EOH!

conv_zones < $NDBOOT >> $OUTFILE

case "$OUTFILE" in
*named*.conf)
	mv $OUTFILE $OUTFILE.$$
	(
	mkhead 
	echo
	echo "options {"
	cat $OPTS
	echo "};"
	echo
	cat <<'!EOL!'
// this is generated by: rndc-confgen -a
include "/etc/rndc.key";

controls {
	inet 127.0.0.1 port 953
	allow { 127.0.0.1; } keys { "rndc-key"; };
};

logging {
	// log what we can		  
	category unmatched { default_syslog; };
	category default { default_syslog; };
};
!EOL!
	cat $OUTFILE.$$
	echo
	echo 'include "primary.zones";'
	if test -s backup.zones; then
		echo 'include "backup.zones";'
	else
		echo '// include "backup.zones";'
	fi	      
	) | sed 's,//;;*,// ,' > $OUTFILE
	;;
esac

# now we want to find out any missing files...
case "$PRIMARY" in
"")
	. ./.env
	here=`pwd`
	# this might take a few iterations
	# but we stop when nothing is happening.
	touch .missing.$$
	DNSDEPS=${DNSDEPS:-$Mydir/dnsdeps.py}
	
	for i in 1 2 3 4 5 6 7 8 9
	do
		eval $DNSDEPS -N ${OUTFILE} 2>&1 |
		grep cannot | sort -u | awk '{ print $NF }' > .missing
		test -s .missing || break
		cmp -s .missing .missing.$$ && break
		(cd $dir && cat $here/.missing | cpio -pdm $here)
		mv .missing .missing.$$
	done
	if test -s .missing; then
		echo Warning: after $i iterations, cannot locate:
		cat .missing
	fi
	;;
esac
