#!/bin/sh

# NAME:
#	dnssec-key-rollover.sh - DNSSec key rollover
#
# SYNOPSIS:
#	dnssec-key-rollover.sh [-K "keydir"] [-k "key"]
#
# DESCRIPTION:
#	DNSSec key rollover is quite straightforward, at least for
#	zone-signing key which is what we operate on by default.
#
#	Rollover of a key-signing key requires interaction with parent
#	zone so will happen much less often (if ever).
#
#	Options:
#
#	-K "keydir"
#		The directory where we keep the "key" to be rolled
#		over. Default is '.'
#
#	-k "key"
#		The "key" to be rolled over.
#		Default is the most recent zone-signing key.
#
#	-I "inactive-date"
#		The date on which to inactivate the current "key".
#		Default is one month hence.
#
#	-D "delete-date"
#		The date on which old "key" is removed from the zone.
#		Default is one month after "inactive-date".
#
#	-n	Do nothing, just say what we would do.
#
#	After setting the dates on the current "key", we will generate
#	a successor key.
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: dnssec-key-rollover.sh,v 1.1 2022/10/07 16:56:22 sjg Exp $
#
#	@(#) Copyright (c) 2022 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

k_date() {
    local jump=$1 Y=${2:-`date +%Y%m%d`} m=$3 d=$4

    case "$Y" in
    ??????*)
	m=${m:-`expr $Y : '....\(..\).*'`}
        d=${d:-`expr $Y : '......\(..\).*'`}
        Y=`expr $Y : '\(....\).*'`
        ;;
    esac
    d=${d:-01}
    if [ ${d#0} -gt 15 ]; then
        jump=$(($jump + 1))
    fi
    m=$((${m#0} + $jump))
    if [ $m -gt 12 ]; then
        Y=$(($Y + 1))
        m=$((m - 12))
    fi
    if [ $m -lt 10 ]; then
        m=0$m
    fi
    echo ${Y}${m}01
}

opt_str=D:I:K:k:n
opt_K=.
. setopts.sh

cd $opt_K || exit 1
opt_k=${opt_k:-`grep -l zone-signing K*key | xargs 'ls' -1t | head -1`}
opt_I=${opt_I:-`k_date 1`}
opt_D=${opt_D:-`k_date 1 $opt_I`}

if [ -z "$opt_n" ]; then
    ECHO=
else
    ECHO=echo
fi

$ECHO dnssec-settime -I $opt_I -D $opt_D $opt_k
$ECHO dnssec-keygen -S $opt_k
